package com.iconplus.smartproc.controller;

import com.iconplus.smartproc.helper.model.EmptyResponse;
import com.iconplus.smartproc.model.request.DrpDokumenRequest;
import com.iconplus.smartproc.model.request.DrpPengadaanRequest;
import com.iconplus.smartproc.model.request.DrpRequest;
import com.iconplus.smartproc.model.response.*;
import com.iconplus.smartproc.service.drp.*;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

@CrossOrigin(origins = "${fe.server}", allowCredentials = "true")
@RestController
@RequestMapping("/api/drp")
public class DrpController {

    private final GetDrpService getDrpService;
    private final GetListDrpService getListDrpService;
    private final PostCreateDrpService postCreateDrpService;
    private final PostCreateDrpPengadaanService postCreateDrpPengadaanService;
    private final GetListDrpPengadaanService getListDrpPengadaanService;
    private final PostDrpUploadDokumenRKAPService postDrpUploadDokumenRKAPService;
    private final PostDrpUploadDokumenPendukungService postDrpUploadDokumenPendukungService;
    private final PostCreateTahunDrpService postCreateTahunDrpService;

    public DrpController(GetDrpService getDrpService,
                         GetListDrpService getListDrpService,
                         PostCreateDrpService postCreateDrpService,
                         PostCreateDrpPengadaanService postCreateDrpPengadaanService,
                         GetListDrpPengadaanService getListDrpPengadaanService,
                         PostDrpUploadDokumenRKAPService postDrpUploadDokumenRKAPService,
                         PostCreateTahunDrpService postCreateTahunDrpService,
                         PostDrpUploadDokumenPendukungService postDrpUploadDokumenPendukungService) {
        this.getDrpService = getDrpService;
        this.getListDrpService = getListDrpService;
        this.postCreateDrpService = postCreateDrpService;
        this.postCreateDrpPengadaanService = postCreateDrpPengadaanService;
        this.getListDrpPengadaanService = getListDrpPengadaanService;
        this.postDrpUploadDokumenRKAPService = postDrpUploadDokumenRKAPService;
        this.postDrpUploadDokumenPendukungService = postDrpUploadDokumenPendukungService;
        this.postCreateTahunDrpService = postCreateTahunDrpService;
    }

    @GetMapping
    public GetListDrpResponse getListDrp(@RequestParam(name = "search", required = false) String search,
                                              @RequestParam(name = "page", defaultValue = "1") Integer page,
                                              @RequestParam(name = "size", defaultValue = "5") Integer size){

        Pageable pageable = PageRequest.of((page - 1), size);
        DrpRequest drpRequest = DrpRequest.builder()
                .search(search)
                .pageable(pageable)
                .build();

        return getListDrpService.execute(drpRequest);
    }


    @PostMapping("/tahun")
    public DrpResponse createTahunDrp(@RequestBody DrpRequest drpRequest) {
        return postCreateTahunDrpService.execute(drpRequest);
    }


    @PostMapping("/upload/rkap")
    public DrpDokumenResponse uploadDokumenRkap(@RequestParam(name = "jenisDokumenId") Long jenisDokumenId,
                                                @RequestParam(name = "keterangan") String keterangan,
                                                @RequestParam(name = "file") MultipartFile file) throws IOException {
        return postDrpUploadDokumenRKAPService.execute(DrpDokumenRequest.builder()
                .jenisDokumenId(jenisDokumenId)
                .keterangan(keterangan)
                .file(file)
                .build());
    }

    @PostMapping("/upload/pendukung")
    public DrpDokumenResponse uploadDokumenPendukung(@RequestParam(name = "jenisDokumenId") Long jenisDokumenId,
                                                     @RequestParam(name = "keterangan") String keterangan,
                                                     @RequestParam(name = "file") MultipartFile file) throws IOException {
        return postDrpUploadDokumenPendukungService.execute(DrpDokumenRequest.builder()
                .jenisDokumenId(jenisDokumenId)
                .keterangan(keterangan)
                .file(file)
                .build());
    }

    @PostMapping
    public DrpResponse createDrp(@RequestBody DrpRequest drpRequest) {
        return postCreateDrpService.execute(drpRequest);
    }

    @GetMapping("/{id}")
    public DrpResponse getDrp(@PathVariable Long id) {
        return getDrpService.execute(DrpRequest.builder()
                .id(id)
                .build());
    }

    @PostMapping("/pengadaan")
    public DrpPengadaanResponse createDrpPengadaan(@RequestBody DrpPengadaanRequest drpPengadaanRequest) {
        return postCreateDrpPengadaanService.execute(drpPengadaanRequest);
    }

    @GetMapping("/pengadaan")
    public GetListDrpPengadaanResponse getDrpPengadaan(@RequestParam(name = "page", defaultValue = "1") Integer page,
                                                       @RequestParam(name = "size", defaultValue = "5") Integer size){

        Pageable pageable = PageRequest.of((page - 1), size);
        DrpPengadaanRequest drpPengadaanRequest = DrpPengadaanRequest.builder()
                .pageable(pageable)
                .build();
        return getListDrpPengadaanService.execute(drpPengadaanRequest);
    }

}
