package com.iconplus.smartproc.repository;

import com.iconplus.smartproc.model.entity.Roles;
import com.iconplus.smartproc.model.projection.RolesView;
import com.iconplus.smartproc.model.projection.UserRoleView;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.Optional;

@Repository
public interface RolesRepository extends JpaRepository<Roles, Long> {

    @Query(value = "SELECT u.id as id, " +
            "u.username as username, " +
            "u.password as password, " +
            "u.roleId as roleId, " +
            "r.role as role " +
            "FROM Users u " +
            "join Roles r on r.id=u.roleId " +
            "WHERE u.isDelete = false " +
            "AND r.isDelete = false " +
            "AND u.username = :userName ")
    Optional<UserRoleView> getUserRoleByUserId(String userName);

    @Query(value = "SELECT r.id as id, " +
            "r.role as role, " +
            "r.keterangan as keterangan, " +
            "r.isActive as isActive, " +
            "r.isDelete as isDelete " +
            "FROM Roles r " +
            "WHERE r.isDelete = false " +
            "AND (:search = '' " +
            "or (UPPER(r.role) like :search OR UPPER(r.keterangan) like :search)) " +
            "ORDER BY r.id")
    Page<RolesView> findAllRolesANdDeletedFalse(String search, Pageable pageable);

    Optional<Roles> findByIdAndIsDeleteFalse(Long id);

    Optional<Roles> findByRoleAndIsDeleteFalse(String role);
    
}
