package com.iconplus.smartproc.service.authentication;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.model.EmptyResponse;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.ForgotPasswordRequest;
import com.iconplus.smartproc.repository.UsersRepository;
import com.iconplus.smartproc.service.CommonService;
import com.iconplus.smartproc.util.Constants;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpStatus;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

@Service
public class ForgotPasswordService implements BaseService<ForgotPasswordRequest, EmptyResponse> {

    private UsersRepository usersRepository;
    private CommonService commonService;
    public ForgotPasswordService(UsersRepository usersRepository,
                                 CommonService commonService) {
        this.usersRepository = usersRepository;
        this.commonService = commonService;
    }

    @Override
    public EmptyResponse execute(ForgotPasswordRequest input) {

        var users = usersRepository.getUsersByEmailAndExpiredTime(input.getRememberId())
                .orElseThrow(() -> new BusinessException(Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "Remember-id", input.getRememberId())));

        String newPassword = commonService.getPassword(input.getNewPassword());
        String confirmationPassword = commonService.getPassword(input.getConfirmationPassword());

        if (!StringUtils.equalsIgnoreCase(newPassword, confirmationPassword)) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10007,
                    Constants.ERR_TTL_10007,
                    Constants.ERR_MSG_10007);
        }

        BCryptPasswordEncoder bCryptPasswordEncoder = new BCryptPasswordEncoder();
        users.setPassword(bCryptPasswordEncoder.encode(newPassword));
        usersRepository.save(users);

        return new EmptyResponse();
    }
}
