package com.iconplus.smartproc.service.authentication;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.InitForgotPasswordRequest;
import com.iconplus.smartproc.model.response.InitForgotPasswordResponse;
import com.iconplus.smartproc.repository.UsersRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.UUID;

@Service
public class InitForgotPasswordService implements BaseService<InitForgotPasswordRequest, InitForgotPasswordResponse> {

    @Value("${config.forgot-password.expired-time:1440}")
    private Integer expiredTime;

    private final UsersRepository usersRepository;

    public InitForgotPasswordService(UsersRepository usersRepository) {
        this.usersRepository = usersRepository;
    }

    @Override
    public InitForgotPasswordResponse execute(InitForgotPasswordRequest input) {

        var users = usersRepository.findByEmailAndIsDeleteFalse(input.getEmail())
                .orElseThrow(() -> new BusinessException(Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "User", input.getEmail())));

        var expirationTime = Timestamp.from(Instant.now().plusMillis(expiredTime * (long) 60000));

        users.setRememberId(UUID.randomUUID().toString());
        users.setExpiredTime(expirationTime);
        usersRepository.save(users);

        return InitForgotPasswordResponse.builder()
                .rememberId(users.getRememberId())
                .build();
    }
}
