package com.iconplus.smartproc.service.bidang;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.BidangView;
import com.iconplus.smartproc.model.request.BidangRequest;
import com.iconplus.smartproc.model.response.BidangResponse;
import com.iconplus.smartproc.model.response.GetListBidangResponse;
import com.iconplus.smartproc.repository.BidangRepository;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListBidangService implements BaseService<BidangRequest, GetListBidangResponse> {

    private BidangRepository bidangRepository;

    public GetListBidangService(BidangRepository bidangRepository) {
        this.bidangRepository = bidangRepository;
    }

    @Override
    public GetListBidangResponse execute(BidangRequest input) {
        validateRequest(input);
        List<BidangResponse> bidangResponseList = new ArrayList<>();
        var bidangViews= bidangRepository.getListBidang(input.getSearch(), input.getPageable());

        for (BidangView bidangView : bidangViews) {
            BidangResponse bidangResponse = BidangResponse.builder()
                    .id(bidangView.getId())
                    .instansiId(bidangView.getInstansiId())
                    .instansi(bidangView.getInstansi())
                    .bidang(bidangView.getBidang())
                    .keterangan(bidangView.getKeterangan())
                    .isActive(bidangView.getIsActive())
                    .build();
            bidangResponseList.add(bidangResponse);
        }

        return GetListBidangResponse.builder()
                .data(bidangResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(bidangViews.getTotalPages())
                        .totalRecords(bidangViews.getTotalElements())
                        .isFirstPage(bidangViews.isFirst())
                        .isLastPage(bidangViews.isLast())
                        .build())
                .build();
    }

    private void validateRequest(BidangRequest input) {
        if (StringUtils.isNotBlank(input.getSearch())) {
            input.setSearch('%'+ input.getSearch().toUpperCase()+'%');
        }
    }
}
