package com.iconplus.smartproc.service.drp;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.DrpView;
import com.iconplus.smartproc.model.request.DrpRequest;
import com.iconplus.smartproc.model.response.DrpResponse;
import com.iconplus.smartproc.model.response.GetListDrpResponse;
import com.iconplus.smartproc.repository.DrpRepository;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListDrpService implements BaseService<DrpRequest, GetListDrpResponse> {

    private final DrpRepository drpRepository;

    public GetListDrpService(DrpRepository drpRepository) {
        this.drpRepository = drpRepository;
    }

    @Override
    public GetListDrpResponse execute(DrpRequest input) {

        List<DrpResponse> drpResponseList = new ArrayList<>();

        var drpViews = drpRepository.findByIsDeleteFalse(input.getApproveDate(), input.getPageable());
        for (DrpView drpView : drpViews) {
            DrpResponse drpResponse = DrpResponse.builder()
                    .id(drpView.getId())
                    .tahun(drpView.getTahun())
                    .approveStatus(drpView.getApproveStatus())
                    .approveDate(drpView.getApproveDate())
                    .isActive(drpView.getIsActive())
                    .isDelete(drpView.getIsDelete())
                    .build();
            drpResponseList.add(drpResponse);
        }

        return GetListDrpResponse.builder()
                .data(drpResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(drpViews.getTotalPages())
                        .totalRecords(drpViews.getTotalElements())
                        .isFirstPage(drpViews.isFirst())
                        .isLastPage(drpViews.isLast())
                        .build())
                .build();
    }
}
