package com.iconplus.smartproc.service.drp;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.Drp;
import com.iconplus.smartproc.model.entity.DrpDokumen;
import com.iconplus.smartproc.model.entity.DrpPengadaanDokumen;
import com.iconplus.smartproc.model.request.DrpDokumenRequest;
import com.iconplus.smartproc.model.request.DrpPengadaanDokumenRequest;
import com.iconplus.smartproc.model.request.DrpRequest;
import com.iconplus.smartproc.model.response.DrpResponse;
import com.iconplus.smartproc.repository.DrpDokumenRepository;
import com.iconplus.smartproc.repository.DrpPengadaanDokumenRepository;
import com.iconplus.smartproc.repository.DrpRepository;
import com.iconplus.smartproc.util.Constants;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Service
public class PostCreateDrpService implements BaseService<DrpRequest, DrpResponse> {

    private final DrpRepository drpRepository;
    private final DrpDokumenRepository drpDokumenRepository;
    private final DrpPengadaanDokumenRepository drpPengadaanDokumenRepository;

    public PostCreateDrpService (DrpRepository drpRepository,
                                 DrpDokumenRepository drpDokumenRepository,
                                 DrpPengadaanDokumenRepository drpPengadaanDokumenRepository) {
        this.drpRepository = drpRepository;
        this.drpDokumenRepository = drpDokumenRepository;
        this.drpPengadaanDokumenRepository = drpPengadaanDokumenRepository;
    }

    @Override
    public DrpResponse execute(DrpRequest input) {

        var drp = drpRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "DRP", input.getId())));

        drp.setApproveStatus(input.getApproveStatus());
        drp.setIsActive(input.getIsActive());

        var drpSaved = drpRepository.save(drp);

        List<DrpDokumen> drpDokumenList = new ArrayList<>();
        if (ObjectUtils.isNotEmpty(input.getDataDrpDokumen())) {
            for (DrpDokumenRequest drpDokumenRequest : input.getDataDrpDokumen()) {
                DrpDokumen drpDokumen = DrpDokumen.builder()
                        .drpId(drpSaved.getId())
                        .jenisDokumenId(drpDokumenRequest.getJenisDokumenId())
                        .filename(drpDokumenRequest.getFilename())
                        .keterangan(drpDokumenRequest.getKeterangan())
                        .isDelete(false)
                        .build();
                drpDokumenList.add(drpDokumen);
            }

        }

        List<DrpPengadaanDokumen> drpPengadaanDokumenList = new ArrayList<>();
        if (ObjectUtils.isNotEmpty(input.getDataDrpPengadaanDokumen())) {
            for (DrpPengadaanDokumenRequest drpPengadaanDokumenRequest : input.getDataDrpPengadaanDokumen()) {
                DrpPengadaanDokumen drpPengadaanDokumen = DrpPengadaanDokumen.builder()
                        .drpId(drpSaved.getId())
                        .jenisDokumenId(drpPengadaanDokumenRequest.getJenisDokumenId())
                        .filename(drpPengadaanDokumenRequest.getFilename())
                        .keterangan(drpPengadaanDokumenRequest.getKeterangan())
                        .isDelete(false)
                        .build();
                drpPengadaanDokumenList.add(drpPengadaanDokumen);
            }
        }

        drpDokumenRepository.saveAll(drpDokumenList);
        drpPengadaanDokumenRepository.saveAll(drpPengadaanDokumenList);

        return DrpResponse.builder()
                .id(drpSaved.getId())
                .build();
    }
}
