package com.iconplus.smartproc.service.instansi;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.InstansiRequest;
import com.iconplus.smartproc.model.response.InstansiResponse;
import com.iconplus.smartproc.repository.InstansiRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.stereotype.Service;

@Service
public class GetInstansiByIdService implements BaseService<InstansiRequest, InstansiResponse> {

    private InstansiRepository instansiRepository;

    public GetInstansiByIdService(InstansiRepository instansiRepository) {
        this.instansiRepository = instansiRepository;
    }

    @Override
    public InstansiResponse execute(InstansiRequest input) {

        var instansi= instansiRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "instansi", input.getId())));

        return InstansiResponse.builder()
                .id(instansi.getId())
                .instansi(instansi.getInstansi())
                .alamat(instansi.getAlamat())
                .keterangan(instansi.getKeterangan())
                .isActive(instansi.getIsActive())
                .isDelete(instansi.getIsDelete())
                .build();
    }
}
