package com.iconplus.smartproc.service.jenisanggaran;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.JenisAnggaranRequest;
import com.iconplus.smartproc.model.response.JenisAnggaranResponse;
import com.iconplus.smartproc.repository.JenisAnggaranRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;

@Service
@Slf4j
public class PutUpdateJenisAnggaranService implements BaseService<JenisAnggaranRequest, JenisAnggaranResponse> {

    private JenisAnggaranRepository jenisAnggaranRepository;

    public PutUpdateJenisAnggaranService(JenisAnggaranRepository jenisAnggaranRepository) {
        this.jenisAnggaranRepository = jenisAnggaranRepository;
    }

    @Override
    public JenisAnggaranResponse execute(JenisAnggaranRequest input) {

        var jenisAnggaran = jenisAnggaranRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "Jenis Anggaran", input.getId())));

        jenisAnggaran.setJenisAnggaran(input.getJenisAnggaran());
        jenisAnggaran.setKeterangan(input.getKeterangan());
        jenisAnggaran.setIsDelete(BooleanUtils.isTrue(input.getIsDelete()));
        jenisAnggaran.setLastUpdate(new Timestamp(System.currentTimeMillis()));
        jenisAnggaranRepository.save(jenisAnggaran);
        log.info("Success edit jenis anggaran id {}", jenisAnggaran.getId());
        return JenisAnggaranResponse.builder()
                .id(jenisAnggaran.getId())
                .build();
    }
}
