package com.iconplus.smartproc.service.jenispengadaan;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.JenisPengadaan;
import com.iconplus.smartproc.model.request.JenisPengadaanRequest;
import com.iconplus.smartproc.model.response.JenisPengadaanResponse;
import com.iconplus.smartproc.repository.JenisPengadaanRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
public class PostCreateJenisPengadaanService implements BaseService<JenisPengadaanRequest, JenisPengadaanResponse> {

    private JenisPengadaanRepository jenisPengadaanRepository;
    public PostCreateJenisPengadaanService(JenisPengadaanRepository jenisPengadaanRepository) {
        this.jenisPengadaanRepository = jenisPengadaanRepository;
    }

    @Override
    public JenisPengadaanResponse execute(JenisPengadaanRequest input) {
        var jenisPengadaan = jenisPengadaanRepository.findByJenisPengadaanAndIsDeleteFalse(input.getJenisPengadaan());
        if (jenisPengadaan.isPresent()) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10002,
                    Constants.ERR_TTL_10002,
                    String.format(Constants.ERR_MSG_10002, "Jenis Pengadaan", input.getJenisPengadaan()));
        }

        JenisPengadaan jenisPengadaanEntity = JenisPengadaan.builder()
                .jenisPengadaan(input.getJenisPengadaan())
                .keterangan(input.getKeterangan())
                .isActive(input.getIsActive())
                .isDelete(false)
                .build();
        var result = jenisPengadaanRepository.save(jenisPengadaanEntity);
        return JenisPengadaanResponse.builder()
                .id(result.getId())
                .build();
    }
}
