package com.iconplus.smartproc.service.lokasi;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.model.EmptyResponse;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.LokasiRequest;
import com.iconplus.smartproc.repository.LokasiRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class DeleteLokasiService implements BaseService<LokasiRequest, EmptyResponse> {

    private LokasiRepository lokasiRepository;

    public DeleteLokasiService(LokasiRepository lokasiRepository) {
        this.lokasiRepository = lokasiRepository;
    }


    @Override
    public EmptyResponse execute(LokasiRequest input) {

        var lokasi = lokasiRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "Lokasi", input.getId())));

        lokasi.setIsDelete(true);
        lokasiRepository.save(lokasi);
        log.info("success delete jenis lokasi id {}", lokasi.getIsDelete());
        return new EmptyResponse();
    }
}
