package com.iconplus.smartproc.service.lokasi;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.LokasiView;
import com.iconplus.smartproc.model.request.LokasiRequest;
import com.iconplus.smartproc.model.response.GetListLokasiResponse;
import com.iconplus.smartproc.model.response.LokasiResponse;
import com.iconplus.smartproc.repository.LokasiRepository;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListLokasiService implements BaseService<LokasiRequest, GetListLokasiResponse> {

    private LokasiRepository lokasiRepository;

    public GetListLokasiService(LokasiRepository lokasiRepository) {
        this.lokasiRepository = lokasiRepository;
    }

    @Override
    public GetListLokasiResponse execute(LokasiRequest input) {
        validateRequest(input);
        List<LokasiResponse> lokasiResponseList = new ArrayList<>();
        Page<LokasiView> lokasiViews = lokasiRepository.getAllLokasi(input.getSearch(), input.getPageable());

        for (LokasiView lokasiView : lokasiViews) {
            LokasiResponse lokasiResponse = LokasiResponse.builder()
                    .id(lokasiView.getId())
                    .lokasi(lokasiView.getLokasi())
                    .keterangan(lokasiView.getKeterangan())
                    .isActive(lokasiView.getIsActive())
                    .isDelete(lokasiView.getIsDelete())
                    .build();
            lokasiResponseList.add(lokasiResponse);
        }

        return GetListLokasiResponse.builder()
                .data(lokasiResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(lokasiViews.getTotalPages())
                        .totalRecords(lokasiViews.getTotalElements())
                        .isFirstPage(lokasiViews.isFirst())
                        .isLastPage(lokasiViews.isLast())
                        .build())
                .build();
    }

    private void validateRequest(LokasiRequest input) {
        if (StringUtils.isNotBlank(input.getSearch())) {
            input.setSearch('%'+ input.getSearch().toUpperCase()+'%');
        }
    }
}
