package com.iconplus.smartproc.service.manajemenuser;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.UsersRequest;
import com.iconplus.smartproc.model.response.UsersResponse;
import com.iconplus.smartproc.repository.UsersRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
public class GetUserByIdService implements BaseService<UsersRequest, UsersResponse> {

    private UsersRepository usersRepository;

    public GetUserByIdService(UsersRepository usersRepository) {
        this.usersRepository = usersRepository;
    }

    @Override
    public UsersResponse execute(UsersRequest input) {

        var userRoleView = usersRepository.getUserByIdAndDeletedFase(input.getId())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "User", input.getId())));

        return UsersResponse.builder()
                .id(userRoleView.getId())
                .nama(userRoleView.getNama())
                .username(userRoleView.getUsername())
                .jabatanId(userRoleView.getJabatanId())
                .jabatan(userRoleView.getJabatan())
                .roleId(userRoleView.getRoleId())
                .role(userRoleView.getRole())
                .bidangId(userRoleView.getBidangId())
                .bidang(userRoleView.getBidang())
                .instansiId(userRoleView.getInstansiId())
                .instansi(userRoleView.getInstansi())
                .email(userRoleView.getEmail())
                .telepon(userRoleView.getTelepon())
                .isActive(userRoleView.getIsActive())
                .build();
    }
}
