package com.iconplus.smartproc.service.manajemenuser;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.UsersRequest;
import com.iconplus.smartproc.model.response.UsersResponse;
import com.iconplus.smartproc.repository.UsersRepository;
import com.iconplus.smartproc.service.CommonService;
import com.iconplus.smartproc.util.Constants;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
public class PutUpdateUserService implements BaseService<UsersRequest, UsersResponse> {

    private UsersRepository usersRepository;
    private CommonService commonService;

    public PutUpdateUserService(UsersRepository usersRepository,
                                 CommonService commonService) {
        this.usersRepository = usersRepository;
        this.commonService = commonService;
    }
    @Override
    public UsersResponse execute(UsersRequest input) {

        var users = usersRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "User", input.getId())));

        commonService.validateUserRequst(input);

        users.setInstansiId(input.getInstansiId());
        users.setBidangId(input.getBidangId());
        users.setNama(input.getNama());
        users.setUsername(input.getUsername());
        users.setJabatanId(input.getJabatanId());
        users.setRoleId(input.getRoleId());
        users.setEmail(input.getEmail());
        users.setTelepon(input.getTelepon());
        users.setPassword(input.getPassword());
        users.setIsActive(input.getIsActive());
        users.setIsDelete(false);
        var result = usersRepository.save(users);
        return UsersResponse.builder()
                .id(result.getId())
                .build();
    }
}
