package com.iconplus.smartproc.service.metodepenyampaian;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.MetodePenyampaian;
import com.iconplus.smartproc.model.request.MetodePenyampaianRequest;
import com.iconplus.smartproc.model.response.MetodePenyampaianResponse;
import com.iconplus.smartproc.repository.MetodePenyampaianRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class PostCreateMetodePenyampaianService implements BaseService<MetodePenyampaianRequest, MetodePenyampaianResponse> {

    private final MetodePenyampaianRepository metodePenyampaianRepository;

    public PostCreateMetodePenyampaianService(MetodePenyampaianRepository metodePenyampaianRepository) {
        this.metodePenyampaianRepository = metodePenyampaianRepository;
    }

    @Override
    public MetodePenyampaianResponse execute(MetodePenyampaianRequest input) {

        var existMetodePenyampaian = metodePenyampaianRepository.findByMetodePenyampaianAndIsDeleteFalse(input.getMetodePenyampaian());
        if (existMetodePenyampaian.isPresent()) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10002,
                    Constants.ERR_TTL_10002,
                    String.format(Constants.ERR_MSG_10002, "Metode Penyampaian", input.getMetodePenyampaian()));
        }

        MetodePenyampaian metodePenyampaian = MetodePenyampaian.builder()
                .metodePenyampaian(input.getMetodePenyampaian())
                .keterangan(input.getKeterangan())
                .isActive(input.getIsActive())
                .isDelete(false)
                .build();

        var result = metodePenyampaianRepository.save(metodePenyampaian);
        log.info("success insert metode penyampaian id : {}", result.getId());
        return MetodePenyampaianResponse.builder()
                .id(result.getId())
                .build();

    }
}