package com.iconplus.smartproc.service.metodepenyampaian;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.MetodePenyampaianRequest;
import com.iconplus.smartproc.model.response.MetodePenyampaianResponse;
import com.iconplus.smartproc.repository.MetodePenyampaianRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class PutUpdateMetodePenyampaianService implements BaseService<MetodePenyampaianRequest, MetodePenyampaianResponse> {

    private MetodePenyampaianRepository metodePenyampaianRepository;

    public PutUpdateMetodePenyampaianService(MetodePenyampaianRepository metodePenyampaianRepository) {
        this.metodePenyampaianRepository = metodePenyampaianRepository;
    }

    @Override
    public MetodePenyampaianResponse execute(MetodePenyampaianRequest input) {

        var metodePenyampaian = metodePenyampaianRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "Metode Penyampaian", input.getId())));

        metodePenyampaian.setMetodePenyampaian(input.getMetodePenyampaian());
        metodePenyampaian.setKeterangan(input.getKeterangan());
        metodePenyampaian.setIsActive(input.getIsActive());
        metodePenyampaianRepository.save(metodePenyampaian);
        log.info("Success edit metode penyampaian id {}", metodePenyampaian.getId());
        return MetodePenyampaianResponse.builder()
                .id(metodePenyampaian.getId())
                .build();
    }
}
