package com.iconplus.smartproc.service.strategipengadaan;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.StrategiPengadaanRequest;
import com.iconplus.smartproc.model.response.StrategiPengadaanResponse;
import com.iconplus.smartproc.repository.StrategiPengadaanRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
public class GetStrategiPengadaanByIdService implements BaseService<StrategiPengadaanRequest, StrategiPengadaanResponse> {

    private final StrategiPengadaanRepository strategiPengadaanRepository;

    public GetStrategiPengadaanByIdService(StrategiPengadaanRepository strategiPengadaanRepository) {
        this.strategiPengadaanRepository = strategiPengadaanRepository;
    }

    @Override
    public StrategiPengadaanResponse execute(StrategiPengadaanRequest input) {
        var strategiPengadaanView = strategiPengadaanRepository.findByIdAndIsDeleteFalse(input.getId());
        if (strategiPengadaanView.isEmpty()) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10001,
                    Constants.ERR_TTL_10001,
                    String.format(Constants.ERR_MSG_10001, "Strategi Pengadaan", input.getId()));
        }

        return StrategiPengadaanResponse.builder()
                .id(strategiPengadaanView.get().getId())
                .strategiPengadaan(strategiPengadaanView.get().getStrategiPengadaan())
                .keterangan(strategiPengadaanView.get().getKeterangan())
                .isActive(strategiPengadaanView.get().getIsActive())
                .isDelete(strategiPengadaanView.get().getIsDelete())
                .build();
    }
}
