package com.iconplus.smartproc.service.sumberdana;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.SumberDanaView;
import com.iconplus.smartproc.model.request.SumberDanaRequest;
import com.iconplus.smartproc.model.response.GetListSumberDanaResponse;
import com.iconplus.smartproc.model.response.SumberDanaResponse;
import com.iconplus.smartproc.repository.SumberDanaRepository;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListSumberDanaService implements BaseService<SumberDanaRequest, GetListSumberDanaResponse> {

    private SumberDanaRepository sumberDanaRepository;

    public GetListSumberDanaService(SumberDanaRepository sumberDanaRepository) {
        this.sumberDanaRepository = sumberDanaRepository;
    }

    @Override
    public GetListSumberDanaResponse execute(SumberDanaRequest input) {
        validateRequest(input);
        List<SumberDanaResponse> sumberDanaResponseList = new ArrayList<>();
        var sumberDanaViews= sumberDanaRepository.getListSumberDana(input.getSearch(), input.getPageable());
        for (SumberDanaView sumberDanaView : sumberDanaViews) {
            SumberDanaResponse sumberDanaResponse = SumberDanaResponse.builder()
                    .id(sumberDanaView.getId())
                    .sumberDana(sumberDanaView.getSumberDana())
                    .keterangan(sumberDanaView.getKeterangan())
                    .isActive(sumberDanaView.getIsActive())
                    .build();
            sumberDanaResponseList.add(sumberDanaResponse);
        }
        return GetListSumberDanaResponse.builder()
                .data(sumberDanaResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(sumberDanaViews.getTotalPages())
                        .totalRecords(sumberDanaViews.getTotalElements())
                        .isFirstPage(sumberDanaViews.isFirst())
                        .isLastPage(sumberDanaViews.isLast())
                        .build())
                .build();
    }

    private void validateRequest(SumberDanaRequest input) {
        if (StringUtils.isNotBlank(input.getSearch())) {
            input.setSearch('%'+ input.getSearch().toUpperCase()+'%');
        }
    }
}
