package com.iconplus.smartproc.service.sumberdana;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.SumberDanaRequest;
import com.iconplus.smartproc.model.response.SumberDanaResponse;
import com.iconplus.smartproc.repository.SumberDanaRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
public class GetSumberDanaService implements BaseService<SumberDanaRequest, SumberDanaResponse> {

    private SumberDanaRepository sumberDanaRepository;

    public GetSumberDanaService(SumberDanaRepository sumberDanaRepository) {
        this.sumberDanaRepository = sumberDanaRepository;
    }

    @Override
    public SumberDanaResponse execute(SumberDanaRequest input) {

        var sumberDana = sumberDanaRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "Sumber Dana", input.getId())));
        return SumberDanaResponse.builder()
                .id(sumberDana.getId())
                .sumberDana(sumberDana.getSumberDana())
                .keterangan(sumberDana.getKeterangan())
                .isActive(sumberDana.getIsActive())
                .isDelete(sumberDana.getIsDelete())
                .build();
    }
}
