package com.iconplus.smartproc.service.sumberdana;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.SumberDana;
import com.iconplus.smartproc.model.request.SumberDanaRequest;
import com.iconplus.smartproc.model.response.SumberDanaResponse;
import com.iconplus.smartproc.repository.SumberDanaRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
public class PostCreateSumberDanaService implements BaseService<SumberDanaRequest, SumberDanaResponse> {

    private SumberDanaRepository sumberDanaRepository;

    public PostCreateSumberDanaService(SumberDanaRepository sumberDanaRepository) {
        this.sumberDanaRepository = sumberDanaRepository;
    }

    @Override
    public SumberDanaResponse execute(SumberDanaRequest input) {

        var sumberDana = sumberDanaRepository.findBySumberDanaAndIsDeleteFalse(input.getSumberDana());
        if (sumberDana.isPresent()) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10002,
                    Constants.ERR_TTL_10002,
                    String.format(Constants.ERR_MSG_10002, "Sumber Dana", input.getSumberDana()));
        }
        SumberDana sumberDanaEntity = SumberDana.builder()
                .sumberDana(input.getSumberDana())
                .keterangan(input.getKeterangan())
                .isActive(input.getIsActive())
                .isDelete(false)
                .build();

        var result = sumberDanaRepository.save(sumberDanaEntity);

        return SumberDanaResponse.builder()
                .id(result.getId())
                .build();
    }
}
