package com.iconplus.smartproc.service.supplypositioningmatrix;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.model.EmptyResponse;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.SupplyPositioningMatrixRequest;
import com.iconplus.smartproc.repository.SupplyPositioningMatrixRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class DeleteSupplyPositioningMatrixService implements BaseService<SupplyPositioningMatrixRequest, EmptyResponse> {

    private SupplyPositioningMatrixRepository supplyPositioningMatrixRepository;

    public DeleteSupplyPositioningMatrixService(SupplyPositioningMatrixRepository supplyPositioningMatrixRepository) {
        this.supplyPositioningMatrixRepository = supplyPositioningMatrixRepository;
    }


    @Override
    public EmptyResponse execute(SupplyPositioningMatrixRequest input) {

        var supplyPositioningMatrix = supplyPositioningMatrixRepository.findByIdAndIsDeleteFalse(input.getId())
                .orElseThrow(() -> new BusinessException(HttpStatus.CONFLICT,
                        Constants.ERR_CODE_10001,
                        Constants.ERR_TTL_10001,
                        String.format(Constants.ERR_MSG_10001, "Supply Posotioning Matrix", input.getId())));

        supplyPositioningMatrix.setIsDelete(true);
        supplyPositioningMatrixRepository.save(supplyPositioningMatrix);
        log.info("success delete supply positioning matrix id {}", supplyPositioningMatrix.getIsDelete());
        return new EmptyResponse();
    }
}
