package com.iconplus.smartproc.service.supplypositioningmatrix;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.SupplyPositioningMatrixView;
import com.iconplus.smartproc.model.request.SupplyPositioningMatrixRequest;
import com.iconplus.smartproc.model.response.GetListSupplyPositioningMatrixResponse;
import com.iconplus.smartproc.model.response.SupplyPositioningMatrixResponse;
import com.iconplus.smartproc.repository.SupplyPositioningMatrixRepository;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListSupplyPositioningMatrixService implements BaseService<SupplyPositioningMatrixRequest, GetListSupplyPositioningMatrixResponse> {

    private SupplyPositioningMatrixRepository supplyPositioningMatrixRepository;

    public GetListSupplyPositioningMatrixService(SupplyPositioningMatrixRepository supplyPositioningMatrixRepository) {
        this.supplyPositioningMatrixRepository = supplyPositioningMatrixRepository;
    }

    @Override
    public GetListSupplyPositioningMatrixResponse execute(SupplyPositioningMatrixRequest input) {
        validateRequest(input);
        List<SupplyPositioningMatrixResponse> supplyPositioningMatrixResponseList = new ArrayList<>();
        Page<SupplyPositioningMatrixView> supplyPositioningMatrixViews = supplyPositioningMatrixRepository.getAllSupplyPositioningMatrix(input.getSearch(), input.getPageable());

        for (SupplyPositioningMatrixView supplyPositioningMatrixView : supplyPositioningMatrixViews) {
            SupplyPositioningMatrixResponse supplyPositioningMatrixResponse = SupplyPositioningMatrixResponse.builder()
                    .id(supplyPositioningMatrixView.getId())
                    .supplyPositioningMatrix(supplyPositioningMatrixView.getSupplyPositioningMatrix())
                    .keterangan(supplyPositioningMatrixView.getKeterangan())
                    .isActive(supplyPositioningMatrixView.getIsActive())
                    .isDelete(supplyPositioningMatrixView.getIsDelete())
                    .build();
            supplyPositioningMatrixResponseList.add(supplyPositioningMatrixResponse);
        }

        return GetListSupplyPositioningMatrixResponse.builder()
                .data(supplyPositioningMatrixResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(supplyPositioningMatrixViews.getTotalPages())
                        .totalRecords(supplyPositioningMatrixViews.getTotalElements())
                        .isFirstPage(supplyPositioningMatrixViews.isFirst())
                        .isLastPage(supplyPositioningMatrixViews.isLast())
                        .build())
                .build();
    }

    private void validateRequest(SupplyPositioningMatrixRequest input) {
        if (StringUtils.isNotBlank(input.getSearch())) {
            input.setSearch('%'+ input.getSearch().toUpperCase()+'%');
        }
    }
}
