package com.iconplus.smartproc.service.supplypositioningmatrix;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.SupplyPositioningMatrix;
import com.iconplus.smartproc.model.request.SupplyPositioningMatrixRequest;
import com.iconplus.smartproc.model.response.SupplyPositioningMatrixResponse;
import com.iconplus.smartproc.repository.SupplyPositioningMatrixRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class PostCreateSupplyPositioningMatrixService implements BaseService<SupplyPositioningMatrixRequest, SupplyPositioningMatrixResponse> {

    private final SupplyPositioningMatrixRepository supplyPositioningMatrixRepository;

    public PostCreateSupplyPositioningMatrixService(SupplyPositioningMatrixRepository supplyPositioningMatrixRepository) {
        this.supplyPositioningMatrixRepository = supplyPositioningMatrixRepository;
    }

    @Override
    public SupplyPositioningMatrixResponse execute(SupplyPositioningMatrixRequest input) {

        var existSupplyPositioningMatrix = supplyPositioningMatrixRepository.findBySupplyPositioningMatrixAndIsDeleteFalse(input.getSupplyPositioningMatrix());
        if (existSupplyPositioningMatrix.isPresent()) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10002,
                    Constants.ERR_TTL_10002,
                    String.format(Constants.ERR_MSG_10002, "Supply Positioning Matrix", input.getSupplyPositioningMatrix()));
        }

        SupplyPositioningMatrix supplyPositioningMatrix = SupplyPositioningMatrix.builder()
                .supplyPositioningMatrix(input.getSupplyPositioningMatrix())
                .keterangan(input.getKeterangan())
                .isActive(input.getIsActive())
                .isDelete(false)
                .build();

        var result = supplyPositioningMatrixRepository.save(supplyPositioningMatrix);
        log.info("success insert supply positioning matrix id : {}", result.getId());
        return SupplyPositioningMatrixResponse.builder()
                .id(result.getId())
                .build();

    }
}