package com.iconplus.smartproc.service.unitinisiator;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.UnitInisiator;
import com.iconplus.smartproc.model.request.UnitInisiatorRequest;
import com.iconplus.smartproc.model.response.UnitInisiatorResponse;
import com.iconplus.smartproc.repository.UnitInisiatorRepository;
import com.iconplus.smartproc.util.Constants;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

@Service
public class PostCreateUnitInisiatorService implements BaseService<UnitInisiatorRequest, UnitInisiatorResponse> {

    private UnitInisiatorRepository unitInisiatorRepository;
    private PostCreateUnitInisiatorService(UnitInisiatorRepository unitInisiatorRepository) {
        this.unitInisiatorRepository = unitInisiatorRepository;
    }

    @Override
    public UnitInisiatorResponse execute(UnitInisiatorRequest input) {

        var unitInisiator = unitInisiatorRepository.findByUnitInisiatorAndIsDeleteFalse(input.getUnitInisiator());
        if (unitInisiator.isPresent()) {
            throw new BusinessException(HttpStatus.CONFLICT,
                    Constants.ERR_CODE_10002,
                    Constants.ERR_TTL_10002,
                    String.format(Constants.ERR_MSG_10002, "Unit Inisiator", input.getUnitInisiator()));
        }

        UnitInisiator unitInisiatorEntity = UnitInisiator.builder()
                .unitInisiator(input.getUnitInisiator())
                .keterangan(input.getKeterangan())
                .isActive(input.getIsActive())
                .isDelete(false)
                .build();
        var result = unitInisiatorRepository.save(unitInisiatorEntity);

        return UnitInisiatorResponse.builder()
                .id(result.getId())
                .build();
    }
}
